/*
 * Copyright (C) 1997-98 Knut Anders Hatlen <kahatlen@online.no>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "pwgen.h"
#include <stdio.h>

void arg_conflict ();

struct option
parse_args (int argc, char **argv)
{
  struct option args;
  int i, j, len = 0;
  char c;

  args.len = -1;
  args.type = -1;
  args.quiet = -1;
  args.help = 0;
  args.version = 0;

  if (argc == 1)
    {
      args.len = PW_DEFAULT_LEN;

      args.type = PW_DEFAULT_TYPE;
      args.quiet = PW_QUIET;
      return args;
    }

  for (i = 1; i < argc; i++)
    {
      for (j = 0; (c = argv[i][j]) != '\0'; j++)
	{
	  switch (c)
	    {
	    case '-':
	      break;
	    case 'l':
	      if (args.type == PW_UCASE || args.type == PW_MCASE)
		arg_conflict ();
	      args.type = PW_LCASE;
	      break;
	    case 'u':
	      if (args.type == PW_LCASE || args.type == PW_MCASE)
		arg_conflict ();
	      args.type = PW_UCASE;
	      break;
	    case 'm':
	      if (args.type == PW_UCASE || args.type == PW_LCASE)
		arg_conflict ();
	      args.type = PW_MCASE;
	      break;
	    case 'q':
	      if (args.quiet == 0)
		arg_conflict ();
	      args.quiet = 1;
	      break;
	    case 'n':
	      if (args.quiet == 1)
		arg_conflict ();
	      args.quiet = 0;
	      break;
	    case 'h':
	      args.help = 1;
	      break;
	    case 'v':
	      args.version = 1;
	      break;
	    default:
	      if (c < '0' || c > '9')
		{
		  printf ("Unknown option %c\n", c);
		  exit (1);
		}
	      else
		{
		  switch (c)
		    {
		    case '0':
		      len = len * 10;
		      break;
		    case '1':
		      len = len * 10 + 1;
		      break;
		    case '2':
		      len = len * 10 + 2;
		      break;
		    case '3':
		      len = len * 10 + 3;
		      break;
		    case '4':
		      len = len * 10 + 4;
		      break;
		    case '5':
		      len = len * 10 + 5;
		      break;
		    case '6':
		      len = len * 10 + 6;
		      break;
		    case '7':
		      len = len * 10 + 7;
		      break;
		    case '8':
		      len = len * 10 + 8;
		      break;
		    case '9':
		      len = len * 10 + 9;
		      break;
		    }
		  args.len = len;
		}
	      break;
	    }
	}
    }
  if (args.len == -1)
    args.len = PW_DEFAULT_LEN;
  if (args.type == -1)
    args.type = PW_DEFAULT_TYPE;
  if (args.quiet == -1)
    args.quiet = PW_QUIET;
  return args;
}

void
arg_conflict ()
{
  fprintf (stderr, "pwgen: conflicting arguments\n");
  exit (1);
}
