/*
 * Copyright (C) 1997-98 Knut Anders Hatlen <kahatlen@online.no>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <sys/time.h>

#include "pwgen.h"

char *
gen_password (int type, unsigned int len)
{
  char *passwd = NULL, ch;
  unsigned int ucl = 0, lcl = 0, mcl = 0, nr = 0, i = 0, j = 0;
  struct timeval tv;

  passwd = (char *) malloc (len * sizeof (char) + 1);

  if (passwd == NULL)
    {
      fprintf (stderr, "Error while allocating memory for password string.\n");
      exit (1);
    }

  for (i = 0; i <= len; i++)
    passwd[i] = '\0';

  gettimeofday (&tv, NULL);
  srand ((tv).tv_sec * 1000000 + (tv).tv_usec);

  switch (type)
    {				/* Check what sort of password we shall generate */
    case PW_MCASE:		/* Mixed case letters */
      ucl = len / 4;
      nr = len / 4;
      lcl = len - (ucl + nr);
      break;
    case PW_LCASE:		/* Lower case only */
      ucl = 0;
      nr = len / 4;
      lcl = len - nr;
      break;
    case PW_UCASE:		/* Upper case only */
      lcl = 0;
      nr = len / 4;
      ucl = len - nr;
      break;
    default:
      fprintf (stderr, "Wrong password type.\n");
      exit (1);
    }

  for (; ucl > 0; ucl--)
    {				/* Insert upper case letters */
      while (passwd[j] != '\0')
	j = (double) (len) * rand () / (RAND_MAX + 1.0);
      ch = (char) (65 + 26.0 * rand () / (RAND_MAX + 1.0));
      passwd[j] = ch;
    }

  for (; lcl > 0; lcl--)
    {				/* Insert lower case letters */
      while (passwd[j] != '\0')
	j = (double) (len) * rand () / (RAND_MAX + 1.0);
      ch = (char) (97 + 26.0 * rand () / (RAND_MAX + 1.0));
      passwd[j] = ch;
    }

  for (; nr > 0; nr--)
    {				/* Insert numbers */
      while (passwd[j] != '\0')
	j = (double) (len) * rand () / (RAND_MAX + 1.0);
      ch = (char) (48 + 10.0 * rand () / (RAND_MAX + 1.0));
      passwd[j] = ch;
    }

  return passwd;
}
